require "kurki/version"
require 'rest-client'
require 'json'
require 'csv'

module Kurki

  def self.url
    return ENV["KURKI_URL"] if ENV["KURKI_URL"][-1] == "/"
    ENV["KURKI_URL"]+"/"
  end

  def self.get_courses
    get(url + "courses")
  end

  def self.get_students(course_id)
    get(url + "courses/#{course_id}/students")
  end

  def self.get_course(id)
    # First because API returns array with one element
    get(url + "courses/#{id}").first
  end

  # Student_ids in form ["12345678", "12345678"]
  def self.set_students(course_id, student_ids)
    post(url + "courses/#{course_id}/students", parse_students(student_ids))
  end

  # Grades in form {"01234567": "3", "01243483": "4"}
  def self.set_grades(course_id, grades)
    post(url + "courses/#{course_id}/students", parse_grades(grades))
  end

  # .csv in two columns, with one header row
  def self.set_grades_from_csv(course_id, file_path, delimiter=",")
    grades = {}
    read_csv(file_path, delimiter).each do |result|
      grades[result[0]] = result[1]
    end
    set_grades course_id, grades
  end


  # Exam points in format:
  # {
  # "123123123" => {
  #   "1" => 1,
  #   "2" => 56
  #   }
  # }
  def self.set_exam_points(course_id, points)
    post(url + "courses/#{course_id}/students", parse_exam_points(points))
  end


  # Exercise points in format:
  # {
  # "123123123" => {
  #     "2" => 1,
  #     "3" => 4
  #   }
  # }
  def self.set_exercise_points(course_id, points)
    post(url + "courses/#{course_id}/students", parse_exercise_points(points))
  end

  private

  def self.parse_exercise_points(points)
    parse(points, lambda {|x,y| {id: x, exercises: y}})
  end

  def self.parse_exam_points(points)
      parse(points, lambda {|x,y| { id: x, exams: y } })
  end

  def self.read_csv(file_path, delimiter)
    CSV.read(file_path, col_sep: delimiter).drop(1)
  end

  def self.parse_students(student_ids)
    parse(student_ids, lambda { |x| {id: x} })
  end

  def self.parse_grades(grades)
    parse(grades, lambda {|x,y| { id: x, arvosana: y } })
  end

  def self.parse(items, exp)
    json = {}
    json["students"] = items.map(&exp)
    json
  end

  def self.post(address, data, params = {})
    return unless data
    params[:authorization] = ENV["KURKI_TOKEN"]

    result = RestClient::Request.execute(
      :url => address, 
      :method => :post, 
      :payload => data.to_json,
      :headers => { params: params, accept: :json, content_type: :json },
      :verify_ssl => false
    )

    JSON.parse result
  end

  def self.get(address, params = {})
    params[:authorization] = ENV["KURKI_TOKEN"]
    
    result = RestClient::Request.execute(
      :url => address, 
      :method => :get, 
      :headers => { params: params, accept: :json, content_type: :json },
      :verify_ssl => false
    )

    JSON.parse result
  end
end
